<?php
namespace Magenest\Salesforce\Block\Adminhtml\System\Config;

use Magenest\Salesforce\Helper\Constant;
use Magenest\Salesforce\Helper\Data as HelperData;
use Magento\Backend\Block\Template\Context;
use Magento\Config\Block\System\Config\Form\Field;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Data\Form\Element\AbstractElement;

/**
 * Class GetAccessTokenButton
 * @package Magenest\Salesforce\Block\Adminhtml\System\Config
 */
class GetAccessTokenButton extends Field
{
    /** @var string */
    protected $_template = "system/config/button/access-token.phtml";

    /** @var string  */
    protected $_buttonLabel = "Get Access Token";

    /**
     * @var string[]
     */
    private $_scopes = [
        'full',
        'refresh_token',
        'offline_access'
    ];

    /** @var ScopeConfigInterface  */
    protected $_scopeConfig;

    /** @var HelperData  */
    protected $_helperData;

    /**
     * GetAccessTokenButton constructor.
     * @param ScopeConfigInterface $scopeConfig
     * @param HelperData $helperData
     * @param Context $context
     * @param array $data
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        HelperData $helperData,
        Context $context,
        array $data = []
    ) {
        $this->_scopeConfig = $scopeConfig;
        $this->_helperData = $helperData;
        parent::__construct($context, $data);
    }

    /**
     * @return GetAccessTokenButton
     */
    protected function _prepareLayout()
    {
        if (!$this->getTemplate()) {
            $this->setTemplate($this->_template);
        }
        return parent::_prepareLayout();
    }

    /**
     * @param AbstractElement $element
     * @return string
     */
    public function render(AbstractElement $element)
    {
        $element->unsScope()->unsCanUseWebsiteValue()->unsCanUseDefaultValue();
        return parent::render($element);
    }

    /**
     * @return string
     */
    public function getButtonLabel()
    {
        return $this->_buttonLabel;
    }

    /**
     * create element for authorization button in store configuration
     * @param AbstractElement $element
     * @return string
     */
    protected function _getElementHtml(AbstractElement $element)
    {
        $this->setData([
            'html_id' => $element->getHtmlId()
        ]);

        return $this->_toHtml();
    }

    /**
     * @return string
     */
    public function getAuthorizationUrl()
    {
        try {
            $salesforceInstanceURL = Constant::INSTANCE_SF_SANDBOX_MODE;
            $connect_environment = $this->_scopeConfig->getValue(Constant::XML_PATH_SALESFORCE_CONNECT_ENVIRONMENT);
            if ($connect_environment === 1 || $connect_environment === "1") {
                $salesforceInstanceURL = Constant::INSTANCE_SF_PRODUCTION_MODE;
            }
            $url = $salesforceInstanceURL . "services/oauth2/authorize";
            $clientId = $this->_scopeConfig->getValue(Constant::XML_PATH_SALESFORCE_CLIENT_ID);
            $redirectUrs = $this->getRedirectUrl();
            $query = [
                "client_id" => $clientId,
                "response_type" => "code",
                "scope"       => implode(" ", $this->_scopes),
                "prompt" => "consent"
            ];
            $endpoint = $url . '?' . http_build_query($query) . "&redirect_uri=$redirectUrs";
            return $endpoint;
        } catch (\Throwable $exception) {
            $this->_logger->debug($exception->getMessage());
        }
        return "#";
    }

    /**
     * @return string
     */
    public function getRedirectUrl()
    {
        return rtrim($this->_helperData->getFrontendUrlByPath("salesforce/oauth/callback"), "/");
    }
}
