<?php

namespace Magenest\Salesforce\Ui\Component\Listing\Columns;

use Magento\Framework\UrlInterface;
use Magento\Framework\View\Element\UiComponent\ContextInterface;
use Magento\Framework\View\Element\UiComponentFactory;
use Magento\Sales\Model\OrderFactory;
use Magento\Ui\Component\Listing\Columns\Column;

/**
 * Class MagentoLink
 * @package Magenest\Salesforce\Ui\Component\Listing\Columns
 */
class MagentoLink extends Column
{
    /** @var UrlInterface */
    protected $urlBuilder;

    /** @var OrderFactory */
    private $_orderFactory;

    /**
     * MagentoLink constructor.
     *
     * @param UrlInterface $urlInterface
     * @param ContextInterface $context
     * @param UiComponentFactory $uiComponentFactory
     * @param OrderFactory $orderFactory
     * @param array $components
     * @param array $data
     */
    public function __construct(
        UrlInterface $urlInterface,
        ContextInterface $context,
        UiComponentFactory $uiComponentFactory,
        OrderFactory $orderFactory,
        array $components = [],
        array $data = []
    ) {
        $this->urlBuilder = $urlInterface;
        $this->_orderFactory = $orderFactory;
        parent::__construct($context, $uiComponentFactory, $components, $data);
    }

    /**
     * @param array $dataSource
     *
     * @return array
     */
    public function prepareDataSource(array $dataSource)
    {
        if (isset($dataSource['data']['items'])) {
            foreach ($dataSource['data']['items'] as & $item) {
                if ($item['magento_id']) {
                    $path = '';
                    $param = 'id';
                    switch ($item['salesforce_table']) {
                        case 'Account':
                        case 'Contact':
                        case 'Lead':
                            $path = 'customer/index/edit/';
                            break;
                        case 'Order':
                        case 'Opportunity':
                            $path = 'sales/order/view/';
                            $param = 'order_id';
                            break;
                        case 'Product2':
                        case 'OrderItem':
                        case 'OpportunityLineItem':
                        case 'PricebookEntry':
                            $path = 'catalog/product/edit/';
                            break;
                        case 'Campaign':
                            $path = 'catalog_rule/promo_catalog/edit/';
                            break;
                    }
                    $additionalTable = [
                        'SHIPPING',
                        'TAX',
                        'CATEGORY'
                    ];
                    $id = $item['magento_id'];
                    if (!in_array($id, $additionalTable)) {
                        $url = $this->urlBuilder->getUrl(
                            $path,
                            [$param => $param === 'order_id' ? $this->getOrderId($id) : $id]
                        );
                        $item['magento_id'] = "<a href='$url' target='_blank'>$id</a>";
                    }
                }
            }
        }
        return $dataSource;
    }

    /**
     * @param $incrementId
     * @return mixed
     */
    private function getOrderId($incrementId)
    {
        return $this->_orderFactory->create()->loadByIncrementId($incrementId)->getId();
    }
}
